<?php
namespace Teknisa\OPL\Modules\Relatorios\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Exception;
use \Setforms2\Shared\ORM\DateTime;
use Teknisa\OPL\Modules\Relatorios\Services\EmissaoPedidosOLService;
use Zeedhi\Framework\DTO\Response\Row;
use Zeedhi\Framework\DTO\Response;


class EmissaoPedidosOLController extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $newCode;
    protected $crudService;
    
    public function __construct($crudService, $parameterBag, EmissaoPedidosOLService $service, $environment, $newCode) {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
		$this->service      = $service;
		$this->environment  = $environment;
		$this->newCode      = $newCode;
    }
    
    public function retornaFloat($valor) {
        if(gettype($valor) === 'string') {
	        $valor = str_replace('.', '', $valor);
            $valor = str_replace(',', '.', $valor);
	        $valor = (float) $valor;
	    }
	    return $valor;
	}
	
	public function procedureVALIDA_ITEMAUTF_OL(DTO\Request $request, DTO\Response $response) {
	    try {
	        $row = $request->getRow();
	        
	        $procedure = $this->service->procedureVALIDA_ITEMAUTF_OL($row);
	        
	        
	        
	        if($procedure["P_ERROS"] == 'S') {
	           
	            $verifica = $this->service->procedureVERIFICA_INCONS_ITAF_OL($row);
	            
	            $response->addDataSet(new DataSet('data', $verifica));
	            
	        }
	        
	        
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	   
	}
	
	public function getCentralCompras(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $centralDeCompras = $this->service->getCentralCompras();
            
            if(!$centralDeCompras) {
                throw new \Exception("CDFILCENTCMP_NOT_REGISTERED");
            }
            $response->addDataSet(new DataSet("data", $centralDeCompras));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function checkIgualdadeItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $result = $this->service->checkIgualdadeItemPedidoOL($row);
            
            if(isset($result[0]["CONTADOR"])) {
                if($result[0]["CONTADOR"] > 0) {
                    throw new \Exception("ITEM_ALREADY_EXISTS_PEDIDO", 500);
                }
            } else {
                throw new \Exception("ERRO_QUERY", 500);
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function insereItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $rowParamFinal = isset($row["FINAL"]) ? $row["FINAL"] : "";
            $rowParamGeral = isset($row["GERAL"]) ? $row["GERAL"] : "";
            
            $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
            
            if($row["QTPROGENPEOL"] == 0) {
                throw new \Exception("QTPROGENPEOL_EQUALS_ZERO", 500);
            } else {
                if(is_bool(strpos($rowParamGeral, "SVOL"))) {
                    $this->service->insereItemPedidoOL($row);
                } else {
                    if(is_bool(strpos($rowParamFinal, "SVOL"))) {
                        $retornoProcedure = $this->service->procedureCarregaPerifili($row);
                                
                        if($retornoProcedure["P_ERRO"] == 'S') {
                            throw new \Exception("Operação bloqueada!<br>Produto: " . $row["NMPRODUTO"] . ",<br>Entrega: " . $row["DTPROGENPEOL"] . ",<br>Quantidade: " . $row["QTPROGENPEOL"] . ",000<br>Produto fora da periodicidade da unidade!", 500);
                        } else {
                            $this->service->insereItemPedidoOL($row);
                        }
                    } else {
                        $this->service->insereItemPedidoOL($row);
                    }
                }
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function excluiItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->service->excluiItemPedidoOL($row);
                $this->service->excluiItemPedidoOLItemSoli($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function alteraDataEntregaItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $aux = 0;
            
            foreach($rows as $row) {
                $row["OLD_DTPROGENPEOL"] = $row["DTPROGENPEOL"];
                $row["DTPROGENPEOL"] = $row["NEW_DTPROGENPEOL"];
                $result = $this->service->checkIgualdadeItemPedidoOL($row);
                
                if(isset($result[0]["CONTADOR"])) {
                    if($result[0]["CONTADOR"] > 0) {
                        throw new \Exception("DTPROGENPEOL_ALREADY_EXISTS_PEDIDO", 500);
                    } else {
                        $this->service->alteraDataEntregaItemPedidoOL($row);
                    }
                } else {
                    throw new \Exception("ERRO_QUERY", 500);
                }
            }
            
            if($aux == 0)
                $this->service->realizaCommit();
            else
                $this->service->realizaRollback();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function alteraMarcaItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $aux = 0;
            
            foreach($rows as $row) {
                $row["CDMARCA"] = $row["NEW_CDMARCA"];
                $this->service->alteraMarcaItemPedidoOL($row);
            }
            
            if($aux == 0)
                $this->service->realizaCommit();
            else
                $this->service->realizaRollback();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function alteraProdutoItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $rowsSemSegMerc = array();
            $rowsSemProdFiliAtivo = array();
            $rowsAlteradas = array();
            
            foreach($rows as $row) {
                $row["CDPRODUTO"] = $row["NEW_CDPRODUTO"];
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["AJUSTE_UNIDADE"] = self::retornaFloat($row["AJUSTE_UNIDADE"]);
                $row["AJUSTE_QUANTIDADE"] = self::retornaFloat($row["AJUSTE_QUANTIDADE"]);
                
                $cdSegMerc = $this->service->getCdSegMercFilial($row["CDFILDEST"]);
            
                if($cdSegMerc) {
                    $res = $this->service->verificaSegMercado($row);
                    
                    if(empty($res) || count($res) == 0) {
                        $rowsSemSegMerc[] = $row;
                        continue;
                    }
                }
                
                $res1 = $this->service->verificaProdFiliAtivo($row);
                
                if(empty($res1) || count($res1) == 0) {
                    $rowsSemProdFiliAtivo[] = $row;
                    continue;
                }
                
                $QTDAJUSTADA = round(($row["QTPROGENPEOL"] + (($row["AJUSTE_UNIDADE"] / 100) * $row["QTPROGENPEOL"])) + 
                (($row["AJUSTE_QUANTIDADE"] / 100) * ($row["QTPROGENPEOL"] + (($row["AJUSTE_UNIDADE"] / 100) * $row["QTPROGENPEOL"]))), 3);
                    
                if((($QTDAJUSTADA - round($QTDAJUSTADA) < 0.1)) && $QTDAJUSTADA > 1) {
                    $QTDAJUSTADA = round($QTDAJUSTADA);
                }
                
                $adqArredProduto = $this->service->adqArredProduto($row);
                $adqSubGrProd = $this->service->adqSubGrProd($row);
                $adqVerifItem = $this->service->adqVerifItem($row);
                $adqProduto1 = $this->service->adqProduto1($row);
                
                if($adqArredProduto[0]["VRARREDCMP"] > 0) {
                    if(($QTDAJUSTADA - (round($QTDAJUSTADA / $adqArredProduto[0]["VRARREDCMP"]) * $adqArredProduto[0]["VRARREDCMP"])) != 0) {
                        $QTDAJUSTADA = ceil(($QTDAJUSTADA / $adqArredProduto[0]["VRARREDCMP"]) * $adqArredProduto[0]["VRARREDCMP"]);
                    }
                }
                
                $dbQtdePedido = null;
                
                if($row["CONSIDERASOL"] == 'S') {
                    $QTDAJUSTADA = null;
                    $adqQtProdOrigOL = $this->service->adqQtProdOrigOL($row);
                    $adqPesoProdOld = $this->service->adqPesoProdOld($row);
                    $adqPesoProdNew = $this->service->adqPesoProdNew($row);
                    
                    if($adqPesoProdOld[0]["VRPESOUNID"] == $adqPesoProdNew[0]["VRPESOUNID"]) {
                        $dbQtdePedido = $adqQtProdOrigOL[0]["QTORIGPROD"];
                    } else {
                        $dbQtdePedido = $row["QTPROGENPEOL"];
                    }
                    
                    $QTDAJUSTADA = round(($dbQtdePedido + (($row["AJUSTE_UNIDADE"] / 100) * $dbQtdePedido)) +
                    (($row["AJUSTE_QUANTIDADE"] / 100) * ($dbQtdePedido + (($row["AJUSTE_UNIDADE"] / 100) * $dbQtdePedido))), 3);
                    
                    if(((($QTDAJUSTADA - round($QTDAJUSTADA)) < 0.1)) && $QTDAJUSTADA > 1) {
                        $QTDAJUSTADA = round($QTDAJUSTADA);
                    }
                    
                    if($adqArredProduto[0]["VRARREDCMP"] > 0) {
                        if(($QTDAJUSTADA - (round($QTDAJUSTADA / $adqArredProduto[0]["VRARREDCMP"]) * $adqArredProduto[0]["VRARREDCMP"])) != 0) {
                            $QTDAJUSTADA = ceil(($QTDAJUSTADA / $adqArredProduto[0]["VRARREDCMP"]) * $adqArredProduto[0]["VRARREDCMP"]);
                        }
                    }
                }
                    
                $row["NEW_QTPROGENPEOL"] = $QTDAJUSTADA;
                
                if(empty($adqVerifItem) && count($adqVerifItem) == 0) {
                    $this->service->acmInsertItemAltAut($row);
                    $this->service->acmInsMovAltItmPexol($row);
                    $this->service->acmDeleteItem($row);
                } else {
                    $this->service->acmUpdateItem($row);
                    $this->service->acmDeleteItem($row);
                }
                    
                if($row["OLD_CDPRODESTO"] != $row["NEW_CDPRODESTO"]) {
                    $adqNrSoli = $this->service->adqNrSoli($row);
                        
                    if(!empty($adqNrSoli) && count($adqNrSoli) > 0) {
                        $adqUpdateitem = $this->service->adqUpdateitem($row, $adqNrSoli[0]);
                        
                        $acmUpdateItemSoli = 0;
                        
                        if(!empty($adqUpdateitem) && count($adqUpdateitem) > 0) {
                            $acmUpdateItemSoli = $this->service->acmUpdateItemSoli($row, $adqNrSoli[0], $adqUpdateitem[0]);
                        }
                        
                        if($acmUpdateItemSoli <= 0) {
                            $adqExisteItemSoliComPedido = $this->service->adqExisteItemSoliComPedido($row, $adqNrSoli[0]);
                                
                            if(empty($adqExisteItemSoliComPedido)) {
                                $acmInsertItemSoli = $this->service->acmInsertItemSoli($row, $adqNrSoli[0]);
                                    
                                $this->service->acmDeleteItemSoli($row, $adqNrSoli[0]);
                            }
                        } else {
                            if($row["OLD_CDPRODESTO"] != $adqProduto1[0]["CDPRODESTO"]) {
                                $this->service->acmDeleteItemSoli($row, $adqNrSoli[0]);
                            }
                        }
                        $adqAtuItemSoli = $this->service->adqAtuItemSoli($row, $adqNrSoli[0]);
                            
                        if(!empty($adqAtuItemSoli)) {
                            $this->service->acmInsMovOpSol($row, $adqAtuItemSoli[0]);
                        }
                    }
                }
            
            $rowsAlteradas[] = $row;
            }
            
            if(count($rowsAlteradas) == 0) {
                $response->addMessage(new Message("NO_ROWS_UPDATED"));
            } else {
                if(count($rowsSemSegMerc) == 0 && count($rowsSemProdFiliAtivo) == 0) {
                    $response->addMessage(new Message("ALL_ROWS_UPDATED"));
                } else if(count($rowsSemSegMerc) > 0 || count($rowsSemProdFiliAtivo) || 0) {
                    $response->addMessage(new Message("SOME_ROWS_UPDATED"));
                }
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function alteraQuantidadeItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["NEW_QTPROGENPEOL"] = self::retornaFloat($row["NEW_QTPROGENPEOL"]);
                $row["OLD_QTPROGENPEOL"] = self::retornaFloat($row["OLD_QTPROGENPEOL"]);
                
                $adqArredProduto = $this->service->adqArredProduto($row);
                $adqProduto1 = $this->service->adqProduto1($row);
                $adqProdOriginal = $this->service->adqProdOriginal($row);
                
                $row["QTORIGPRODPED"] = $adqProdOriginal[0]["QTORIGPRODPED"];
                
                if($adqArredProduto[0]["VRARREDCMP"] > 0) {
                    if(($row["NEW_QTPROGENPEOL"] - (round($row["NEW_QTPROGENPEOL"] / $adqArredProduto[0]["VRARREDCMP"]) * $adqArredProduto[0]["VRARREDCMP"])) != 0) {
                        $row["NEW_QTPROGENPEOL"] = ceil(($row["NEW_QTPROGENPEOL"] / $adqArredProduto[0]["VRARREDCMP"]) * $adqArredProduto[0]["VRARREDCMP"]);
                    }
                }
                
                $this->service->acmInsMovAltItmPexolQtd($row);
                
                $this->service->acmUpdateItemQtd($row);
                
                $adqNrSoli = $this->service->adqNrSoli($row);
                        
                if(!empty($adqNrSoli) && count($adqNrSoli) > 0) {
                    $adqPesqSolicitacao = $this->service->adqPesqSolicitacao($row);
                    $dbQtSoli = 0;
                    
                    if(!empty($adqPesqSolicitacao) && count($adqPesqSolicitacao) > 0) {
                        $adqAtuItemSolicita = $this->service->adqAtuItemSolicita($row, $adqPesqSolicitacao[0]);
                        
                        if($adqAtuItemSolicita[0]["VRFATOSOLI"] > 0) {
                            $dbQtSoli = (($row["NEW_QTPROGENPEOL"] * $adqAtuItemSolicita[0]["VRFATOCONV"]) / $adqAtuItemSolicita[0]["VRFATOSOLI"]) - $adqAtuItemSolicita[0]["QTSOLIPROD"];
                        } else {
                            $dbQtSoli = $row["NEW_QTPROGENPEOL"];
                        }
                        
                        $this->service->acmUpQtSoliProd($adqAtuItemSolicita[0], $dbQtSoli);
                    }
                    
                    if(!empty($adqAtuItemSolicita)) {
                        $this->service->acmInsMovOpSol($row, $adqAtuItemSolicita[0], $dbQtSoli);
                    }
                }
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function incluiItemPedidoLote(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $this->service->inclusaoDePedidoParaLote($rows[0]);
            
            foreach($rows as $row) {
                $this->service->incluiItemPedidoLote($row);
                $this->service->ajusteItemPedidoLote($row);
                $this->service->excluiReservaItemPedidoLote($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function incluiItemPedidoLoteDiversos(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->service->inclusaoDePedidoParaLote($row);
                $this->service->incluiItemPedidoLote($row);
                $this->service->ajusteItemPedidoLote($row);
                $this->service->excluiReservaItemPedidoLote($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function inclusaoReservaItemPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $aux = 0;
            
            foreach($rows as $row) {
                $QUANTESTOQ = null;
                $QUANTRESRV = null;
                $QUANTITEMP = null;
                $CDOLFILIAL = $this->service->verificaOLQueAtendeFilial($row);
                $QUANTESTOQ = $this->service->posicaoEstoqueItemOL($CDOLFILIAL, $row);
                $QUANTRESRV = $this->service->verificaReservaDemaisPedidos($row);
                $QUANTITEMP = self::retornaFloat($row["QTPROGENPEOL"]);
                
                $QTRESERVESTOL = (($QUANTESTOQ - $QUANTRESRV) - $QUANTITEMP);
                
                if($QTRESERVESTOL >= 0) {
                    $rowsAffected = $this->service->incluiReservaItemPedidoOL($row);
                    
                    if($rowsAffected <= 0) {
                        $aux++;
                    }
                } else {
                    $aux++;
                }
            }
            
            if($aux == 0) {
                $this->service->realizaCommit();
            } else {
                throw new \Exception("QTD_INDISPONIVEL_RESERVA", 500);
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function exclusaoReservaItemPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->service->excluiReservaItemPedidoLote($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function baixaManualItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["QTORIGPRODPED"] = self::retornaFloat($row["QTORIGPRODPED"]);
                $this->service->atualizaSituacaoItemPedidoOL($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function checkOperBlockUnblockPer(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $result = $this->service->checkOperBlockUnblockPer();
            
            if(isset($result[0]["CDOPERADOR"])) {
                $response->addDataSet(new DataSet("CDOPERADOR", array($result[0]["CDOPERADOR"])));
            } else {
                $response->addDataSet(new DataSet("CDOPERADOR", array(null)));
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function desbloqueioBloqueioItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->service->desbloqueioBloqueioItemPedidoOL($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function validacaoSenhaED(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $dataSourceName = "operador";
        $conditions = array(array("columnName" => "CDOPERADOR", "operator" => "=", "value" => $row["CDOPERADOR"]));
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        if (md5($row["SENHA"]) != $dataSet[0]["CDSENHAOPERWEB"]) {
            $response->setError(new Error("INVALID_PASSWORD", 1));
        }
    }
    
    public function processoEntregaDiretaItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $adqParametros = $this->service->getAdqParametros($row);
                
                if($adqParametros["IDGERAITEMSOLI"] == 'N') {
                    $this->service->acmSolPontoAPontoItemPedidoED($row);
                } else {
                    $adqItemSoliEd = $this->service->getAdqItemSoliEd($row);
                    
                    if(is_null($adqItemSoliEd)) {
                        $adqPesqItemSoli = $this->service->getAdqPesqItemSoli($row);
                        
                        if(!isset($adqPesqItemSoli["NRSOLICMP"]) || is_null($adqPesqItemSoli["NRSOLICMP"])) {
                            $this->service->acmInsSolicitaItemPedidoED($row);
                            $this->service->acmInsSolPontoaPontoItemPedidoED($row, $adqParametros, null);
                        } else {
                            $this->service->acmInsSolPontoaPontoItemPedidoED($row, $adqParametros, $adqPesqItemSoli);
                            $this->service->acmInsMovopSolItemPedidoED($row, $adqPesqItemSoli);
                        }
                    } else {
                        $this->service->acmSolPontoaPonto2ItemPedidoED($row, $adqItemSoliEd);
                    }
                }
                
                $this->service->excluiItemPedidoOL($row);
                $this->service->excluiItemPedidoOLItemSoli($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function carregaDadosPedido(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $result = $this->service->carregaDadosPedido($row);
            
            $response->addDataSet(new DataSet("opl07200_dados_pedido", $result));
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function salvaDadosPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $this->service->salvaDadosPedido($row);
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function baixaManualPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $this->service->salvaDadosPedido($row);
            
            $itens = $this->service->selecionaItensDoPedido($row);
            
            foreach($itens as $item) {
                $item["NEW_IDBAIXENPEOL"] = $row["NEW_IDBAIXEXOL"];
                $item["NEW_DTBAIXTMPEOL"] = $row["NEW_DTBAIXTMPEOL"];
                $this->service->atualizaSitItemViaPedido($item);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function cancelaEncerraPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $this->service->salvaDadosPedido($row);
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function emAbertoPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $this->service->salvaDadosPedido($row);
            
            $itens = $this->service->selecionaItensDoPedido($row);
            
            foreach($itens as $item) {
                $item["NEW_IDBAIXENPEOL"] = $row["NEW_IDBAIXEXOL"];
                $this->service->atualizaSitItemViaPedido($item);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function exclusaoPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            //$this->service->exclusaoItensPedido($row);
            
            $itens = $this->service->selecionaItensDoPedido($row);
            
            foreach($itens as $item) {
                $item["CDFILIOPELOG"] = $row["CDFILIOPELOG"];
                
                $this->service->excluiItemPedidoOL($item);
                $this->service->excluiItemPedidoOLItemSoli($item);
            }
            
            $this->service->exclusaoPedido($row);
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function inclusaoPedido(DTO\Request\Row $request, DTO\Response $response) {
        try {
            
            $row = $request->getRow();
            $rowDados = $row["DADOS"];
            $rowProdutos = $row["PRODUTOS"];
            $this->service->beginTransaction();
            
            $NEWNRPEDEXOL = $this->service->inclusaoPedido($rowDados);
            
            $this->service->realizaCommit();
            
            $this->service->beginTransaction();
        
            foreach($rowProdutos as $rowProd) {
                $rowProd["NRPEDEXOL"] = $NEWNRPEDEXOL;
                $rowProd["CDLOTENVITOL"] = isset($rowDados["CDLOTENVITOL"]) ? $rowDados["CDLOTENVITOL"] : null;
                $rowProd["CDFILLANC"] = $rowDados["CDFILLANC"];
                $rowProd["CDFILDEST"] = $rowDados["CDFILDEST"];
                $rowProd["QTPROGENPEOL"] = self::retornaFloat($rowProd["QTPROGENPEOL"]);
                $rowProd["CDFILIOPELOG"] = $rowDados["CDFILIOPELOG"];
                
                if($rowProd["QTPROGENPEOL"] > 0) {
                    $this->service->insereItemPedidoOL($rowProd);
                }
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function adqProcedureCarregaPerifili(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $rowParamFinal = isset($row["FINAL"]) ? $row["FINAL"] : "";
            $rowParamGeral = isset($row["GERAL"]) ? $row["GERAL"] : "";
            
            $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
            
            $retornoProcedure = $this->service->procedureCarregaPerifili($row);
            if(is_bool(strpos($rowParamGeral, "SVOL"))) {
                $response->addMessage(new Message("successful"));
            } else {
                if(is_bool(strpos($rowParamFinal, "SVOL"))) {
                    $retornoProcedure = $this->service->procedureCarregaPerifili($row);
                                
                    if($retornoProcedure["P_ERRO"] == 'S') {
                        throw new \Exception("Operação bloqueada!<br>Produto: " . $row["NMPRODUTO"] . ",<br>Entrega: " . $row["DTPROGENPEOL"] . ",<br>Quantidade: " . $row["QTPROGENPEOL"] . ",000<br>Produto fora da periodicidade da unidade!", 500);
                    } else {
                        $response->addMessage(new Message("successful"));
                    }
                } else {
                    $response->addMessage(new Message("successful"));
                }
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function exportarPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $retornoProcedure = $this->service->procedureGeraPedOLExpedicao($row);
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    const msgInconst = array(
        "01" => "Grupo de Produtos não cadastrado.",
        "02" => "Subgrupo dos Produtos da Árvore Original não cadastrado.",
        "04" => "Produto parametrizado para mais de um agrupamento.",
        "05" => "Entrega Direta.",
        "06" => "Item já gerou pedido OL.",
        "08" => "Item consolidado para processo diferente do OL.",
        "09" => "Item não consolidado."
    );
    
    public function adqValidacoes(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
        
            $novaLista = $this->service->procedureGeraNovoCodigo("LISTA");
            
            $produtos = array();
            
            $this->service->insereNovaLista($novaLista, $row);
            
            $adqValidacoes = $this->service->adqValidacoes($novaLista, $row);
            
            foreach($adqValidacoes as $prod) {
                $prod["DSERRO"] = self::msgInconst[$prod["COD_ERROR"]];
                $prod["DTPROGENTR"] = substr($prod["DTPROGENTR"], 0, 10);
                
                array_push($produtos, $prod);
            }
            
            $this->service->deleteNovaLista($novaLista);
            
            $this->service->realizaCommit();
            
            $response->addDataSet(new DataSet("adqValidacoes", $produtos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
        }
    }
    
    public function importacaoSolicitacaoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            $retornoProcedure = $this->service->incluiPedidoOLProcedure($row);
                
            if($retornoProcedure["P_ROWSAFFECT"] == 'S') {
                $this->service->realizaCommit();
            } else {
                throw new \Exception("ERROR_APUR", 500);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
        }
    }
    
    public function alteraQuantidadeItemPedidoOLInline(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = isset($row["QTPROGENPEOL"]) ? self::retornaFloat($row["QTPROGENPEOL"]) : self::retornaFloat($row["changed_QTPROGENPEOL"]);
                $row["NEW_QTPROGENPEOL"] = self::retornaFloat($row["NEW_QTPROGENPEOL"]);
                $row["OLD_QTPROGENPEOL"] = self::retornaFloat($row["OLD_QTPROGENPEOL"]);
                
                $adqArredProduto = $this->service->adqArredProduto($row);
                $adqProduto1 = $this->service->adqProduto1($row);
                $adqProdOriginal = $this->service->adqProdOriginal($row);
                
                $row["QTORIGPRODPED"] = $adqProdOriginal[0]["QTORIGPRODPED"];
                
                if($adqArredProduto[0]["VRARREDCMP"] > 0) {
                    if(($row["NEW_QTPROGENPEOL"] - (round($row["NEW_QTPROGENPEOL"] / $adqArredProduto[0]["VRARREDCMP"]) * $adqArredProduto[0]["VRARREDCMP"])) != 0) {
                        $row["NEW_QTPROGENPEOL"] = ceil(($row["NEW_QTPROGENPEOL"] / $adqArredProduto[0]["VRARREDCMP"]) * $adqArredProduto[0]["VRARREDCMP"]);
                    }
                }
                
                $this->service->acmInsMovAltItmPexolQtd($row);
                
                $this->service->acmUpdateItemQtd($row);
                
                $adqNrSoli = $this->service->adqNrSoli($row);
                        
                if(!empty($adqNrSoli) && count($adqNrSoli) > 0) {
                    $adqPesqSolicitacao = $this->service->adqPesqSolicitacao($row);
                    $dbQtSoli = 0;
                    
                    if(!empty($adqPesqSolicitacao) && count($adqPesqSolicitacao) > 0) {
                        $adqAtuItemSolicita = $this->service->adqAtuItemSolicita($row, $adqPesqSolicitacao[0]);
                        
                        if($adqAtuItemSolicita[0]["VRFATOSOLI"] > 0) {
                            $dbQtSoli = (($row["NEW_QTPROGENPEOL"] * $adqAtuItemSolicita[0]["VRFATOCONV"]) / $adqAtuItemSolicita[0]["VRFATOSOLI"]) - $adqAtuItemSolicita[0]["QTSOLIPROD"];
                        } else {
                            $dbQtSoli = $row["NEW_QTPROGENPEOL"];
                        }
                        
                        $this->service->acmUpQtSoliProd($adqAtuItemSolicita[0], $dbQtSoli);
                    }
                    
                    if(!empty($adqAtuItemSolicita)) {
                        $this->service->acmInsMovOpSol($row, $adqAtuItemSolicita[0], $dbQtSoli);
                    }
                }
                
                /*if($row["NEW_QTPROGENPEOL"] == 0) {
                    $this->service->excluiItemPedidoOL($row);
                    $this->service->excluiItemPedidoOLItemSoli($row);
                } else {
                    $this->service->alteraQuantidadeItemPedidoOL($row);
                }*/
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function bloqueiaPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $adqBloqueiaPedidoOL = $this->service->adqBloqueiaPedidoOL($row);
            
            if(!empty($adqBloqueiaPedidoOL)) {
                foreach($adqBloqueiaPedidoOL as $item) {
                    $this->service->acmBloqueiaItemPedOL($item);
                }
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function getParametro(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $adqParametro = $this->service->adqParametro();
            $response->addDataSet(new DataSet("getParametro", $adqParametro));
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function verificaCdsItemIgualdade($cdsVdItem, $itemPedido, $op, $dbSoma_Reser = null, $adqProdEstoque = null) {
        $valida = false;
        
        foreach($cdsVdItem as $item) {
            if($item["CDFILDEST"] == $itemPedido["CDFILDEST"] && $item["CDFILLANC"] == $itemPedido["CDFILLANC"] && $item["NRPEDEXOL"] == $itemPedido["NRPEDEXOL"] && $item["CDPRODUTO"] == $itemPedido["CDPRODUTO"] && $item["CDMARCA"] == $itemPedido["CDMARCA"] && $item["DTPROGENPEOL"] == $itemPedido["DTPROGENPEOL"]) {
                $valida = true;
                   
                if($op == "adqVdRelacprodol") {
                    $item["RELACPRODOL"] = 'S';
                } else if($op == "adqProdAtCompra") {
                    $item["PRODATCMP"] = 'S';
                } else if($op == "adqProdConv") {
                    $item["PRODCONV"] = 'S';
                } else if($op == "adqProdNPeso") {
                    $item["PRODNPESO"] = 'S';
                } else if($op == "dbEstoque") {
                    $item["ESTOQUE"] = 'S';
                } else if($op == "dbCusto") {
                    $item["PRODNCOT"] = 'S';
                } else if($op == "adqPrClassFisc") {
                    $item["PRCLASSFISC"] = 'S';
                } else if($op == "adqAgrupamento") {
                    $item["AGRUPAMENTO"] = 'S';
                } else if($op == "adqReserva") {
                    $item["RESERVA"] = 'S';
                } else if($op == "QTEST_DISPO_SOMA_RESER") {
                    $item["QTEST_DISPO"] = empty($adqProdEstoque) ? 0 : $adqProdEstoque[0]["ESTOQ_DISPO"];
                    $item["SOMA_RESER"] = $dbSoma_Reser;
                } else if($op == "adqTipoOperOL") {
                    $item["PARAMFISCOL"] = 'S';
                } else if($op == "adqErroFiscalOL") {
                    $item["CRTLFISOL"] = 'S';
                } else if($op == "adqProdAgrup") {
                    $item["PRODAGRUP"] = 'S';
                }
                
                break;
            } else {
                $valida = false;
            }
        }
        
        return $valida;
    }
    
    public function verificaCdsItemIgualdade1($cdsVdItem1, $itemPedido) {
        $valida = false;
        
        foreach($cdsVdItem1 as $cdVdItem) {
            if($cdVdItem["CDFILIOPELOG"] == $itemPedido["CDFILIOPELOG"] && $cdVdItem["CDPRODUTO"] == $itemPedido["CDPRODUTO"] && $cdVdItem["CDMARCA"] == $itemPedido["CDMARCA"] && $cdVdItem["DTPROGENPEOL"] == $itemPedido["DTPROGENPEOL"]) {
                $valida = true;
                break;
            } else {
                $valida = false;
            }
        }
        
        return $valida;
    }
    
    public function getVdItem(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            $centralDeCompras = $this->service->getCentralCompras();
            $stWebService = 'N';
            $cdsVdItem = array();
            $cdsVdItem1 = array();
            $cdsGravaEnvio = array();
            $dbEstDisp = 0;
            $dbSoma_Reser = 0;
            $dbQtItmPed = 0;
            $dbQtItmReserv = 0;
            $dbEstoque = 0;
            $dbCusto = 0;
            $stNrErFisOL = $this->service->procedureGeraNovoCodigo("ERROFISCALOL");
            
            $adqItemPedido = $this->service->adqItemPedido($row);
            
            if(!empty($adqItemPedido)) {
                foreach($adqItemPedido as $itemPedido) {
                    $adqParasupri = $this->service->adqParasupri($centralDeCompras["CDFILCENTCMP"]);
                    $adqVdRelacprodol = $this->service->adqVdRelacprodol($itemPedido);
                    
                    if($stWebService == 'N') {
                        $adqParamIntegra = $this->service->adqParamIntegra($itemPedido);
                        
                        if(!empty($adqParamIntegra)) {
                            $stWebService = 'S';
                        }
                    }
                    //Produto sem relacionamento do OL cadastrado
                    if(empty($adqVdRelacprodol)) {
                        if(empty($cdsVdItem)) {
                            $itemPedido["RELACPRODOL"] = 'S';
                            $cdsVdItem[] = $itemPedido;
                        } else {
                            if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqVdRelacprodol")) {
                                $itemPedido["RELACPRODOL"] = 'S';
                                $cdsVdItem[] = $itemPedido;
                            }
                        }
                    }
                    
                    $adqProdAtCompra = $this->service->adqProdAtCompra($itemPedido);
                    
                    //O produto não está ativo para compra
                    if(empty($adqProdAtCompra)) {
                        if(empty($cdsVdItem)) {
                            $itemPedido["PRODATCMP"] = 'S';
                            $cdsVdItem[] = $itemPedido;
                        } else {
                            if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqProdAtCompra")) {
                                $itemPedido["PRODATCMP"] = 'S';
                                $cdsVdItem[] = $itemPedido;
                            }
                        }
                    }
                    if(!empty($adqVdRelacprodol)) {
                        foreach($adqVdRelacprodol as $vdItem) {
                            $adqProdConv = $this->service->adqProdConv($vdItem);
                            
                            //Produto sem conversão para a unidade de estoque
                            if(empty($adqProdConv)) {
                                if(empty($cdsVdItem)) {
                                    $itemPedido["PRODCONV"] = 'S';
                                    $cdsVdItem[] = $itemPedido;
                                } else {
                                    if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqProdConv")) {
                                        $itemPedido["PRODCONV"] = 'S';
                                        $cdsVdItem[] = $itemPedido;
                                    }
                                }
                            }
                            
                            $adqProdNPeso = $this->service->adqProdNPeso($vdItem);
                            
                            //Produto sem peso unitário cadastrado
                            if(empty($adqProdNPeso)) {
                                if(empty($cdsVdItem)) {
                                    $itemPedido["PRODNPESO"] = 'S';
                                    $cdsVdItem[] = $itemPedido;
                                } else {
                                    if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqProdNPeso")) {
                                        $itemPedido["PRODNPESO"] = 'S';
                                        $cdsVdItem[] = $itemPedido;
                                    }
                                }
                            }
                            
                            $adqFiliAgentTerceiro = $this->service->adqFiliAgentTerceiro($itemPedido);
                            
                            //Localiza a última data de entrada do produto no OL
                            $adqUltimoMovEst = $this->service->adqUltimoMovEst($vdItem, $adqFiliAgentTerceiro[0]);
                            //Produto sem estoque disponível
                            $adqProdEstoque = $this->service->adqProdEstoque($vdItem, $adqFiliAgentTerceiro[0], $adqUltimoMovEst[0]);
                            $adqProdReserva = $this->service->adqProdReserva($itemPedido, $adqFiliAgentTerceiro[0]);
                            $adqReserva = $this->service->adqReserva($itemPedido);
                            
                            if(empty($adqProdEstoque)) {
                                $dbEstDisp = 0;
                            } else {
                                $dbEstDisp = $adqProdEstoque[0]["ESTOQ_DISPO"];
                            }
                            if(empty($adqProdReserva)) {
                                $dbSoma_Reser = 0;
                            } else {
                                $dbSoma_Reser = $adqProdReserva[0]["ESTOQ_RESERVADO"];
                            }
                            if(empty($adqItemPedido)) {
                                $dbQtItmPed = 0;
                            } else {
                                $dbQtItmPed = $itemPedido["QTPROGENPEOL"];
                            }
                            if(empty($adqReserva)) {
                                $dbQtItmReserv = 0;
                            } else {
                                $dbQtItmReserv = $adqReserva[0]["QTRESERVESTOL"];
                            }
                            if($adqParasupri[0]["IDUTRESPEDEXOL"] == 'S') {
                                if($dbQtItmReserv == 0) {
                                    $dbEstoque = $dbEstDisp - $dbSoma_Reser - $dbQtItmPed;
                                    
                                    if($dbEstoque >= 0) {
                                        if($itemPedido["IDBAIXAOLSAI"] == 0) {
                                            if($row["P_GERARESERVA"] == 'S') {
                                                if(empty($adqReserva)) {
                                                    $acmInsReseva = $this->service->acmInsReserva($itemPedido, $adqFiliAgentTerceiro[0]);
                                                }
                                            }
                                        }
                                    }
                                } else {
                                    $dbEstoque = $dbEstDisp - $dbSoma_Reser - $dbQtItmReserv;
                                }
                            }
                            //Produto sem quantidade disponível em estoque
                            if($dbEstoque < 0) {
                                if(empty($cdsVdItem)) {
                                    $itemPedido["ESTOQUE"] = 'S';
                                    $cdsVdItem[] = $itemPedido;
                                } else {
                                    if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "dbEstoque")) {
                                        $itemPedido["ESTOQUE"] = 'S';
                                        $cdsVdItem[] = $itemPedido;
                                    }
                                }
                            }
                            
                            //Produto sem Custo
                            $adqParam = $this->service->adqParam($itemPedido);
                            $adqCusto = $this->service->adqCusto($itemPedido, $vdItem, $adqParam[0]);
                            
                            if(!empty($adqCusto)) {
                                $dbCusto = $adqCusto[0]["VRCUSTO"];
                            }
                            if($dbCusto <= 0) {
                                if(empty($cdsVdItem)) {
                                    $itemPedido["PRODNCOT"] = 'S';
                                    $cdsVdItem[] = $itemPedido;
                                } else {
                                    if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "dbCusto")) {
                                        $itemPedido["PRODNCOT"] = 'S';
                                        $cdsVdItem[] = $itemPedido;
                                    }
                                }
                            }
                            
                            $adqPrClassFisc = $this->service->adqPrClassFisc($vdItem);
                            
                            if(empty($adqPrClassFisc)) {
                                if(empty($cdsVdItem)) {
                                    $itemPedido["PRCLASSFISC"] = 'S';
                                    $cdsVdItem[] = $itemPedido;
                                } else {
                                    if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqPrClassFisc")) {
                                        $itemPedido["PRCLASSFISC"] = 'S';
                                        $cdsVdItem[] = $itemPedido;
                                    }
                                }
                            }
                        }
                    } else {
                        if(empty($cdsVdItem)) {
                            $itemPedido["RELACPRODOL"] = 'S';
                            $cdsVdItem[] = $itemPedido;
                        } else {
                            if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqVdRelacprodol")) {
                                $itemPedido["RELACPRODOL"] = 'S';
                                $cdsVdItem[] = $itemPedido;
                            }
                        }
                    }
                    
                    $adqAgrupamento = $this->service->adqAgrupamento($itemPedido);
                    
                    //Produto sem agrupamento de produtos
                    if(empty($adqAgrupamento)) {
                        if(empty($cdsVdItem)) {
                            $itemPedido["AGRUPAMENTO"] = 'S';
                            $cdsVdItem[] = $itemPedido;
                        } else {
                            if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqAgrupamento")) {
                                $itemPedido["AGRUPAMENTO"] = 'S';
                                $cdsVdItem[] = $itemPedido;
                            }
                        }
                    }
                    if($adqParasupri[0]["IDUTRESPEDEXOL"] == 'S') {
                        $adqReserva = $this->service->adqReserva($itemPedido);
                        
                        //Produto sem reserva cadastrada
                        if(empty($adqReserva)) {
                            if(empty($cdsVdItem)) {
                                $itemPedido["RESERVA"] = 'S';
                                $cdsVdItem[] = $itemPedido;
                            } else {
                                if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqReserva")) {
                                    $itemPedido["RESERVA"] = 'S';
                                    $cdsVdItem[] = $itemPedido;
                                }
                            }
                        }
                    }
                    
                    if(!empty($adqVdRelacprodol) && !empty($adqProdEstoque)) {
                        self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "QTEST_DISPO_SOMA_RESER", $dbSoma_Reser, $adqProdEstoque);
                    }
                    
                    foreach($cdsVdItem as $cdVdItem) {
                        if($cdVdItem["CDFILIOPELOG"] == $itemPedido["CDFILIOPELOG"] && $cdVdItem["CDPRODUTO"] == $itemPedido["CDPRODUTO"] && $cdVdItem["CDMARCA"] == $itemPedido["CDMARCA"] && $cdVdItem["DTPROGENPEOL"] == $itemPedido["DTPROGENPEOL"]) {
                            $cdVdItem["SOMA_RESER"] = $dbSoma_Reser;
                        }
                    }
                    
                    if(!($row["P_SUBGRUPO"] == 'N' || strlen($row["P_SUBGRUPO"]) > 12)) {
                        if(!self::verificaCdsItemIgualdade1($cdsVdItem1, $itemPedido)) {
                            $cdsVdItem1[] = $itemPedido;
                        }
                    }
                    
                    $adqTipoOperOL = $this->service->adqTipoOperOL($itemPedido);
                    
                    if($adqParasupri[0]["IDUTCTRLFISOL"] == 'S' /** && UTILIZA CONTROLE FISCAL -> dtmRotiFiscal.Utiliza_ControleFiscal(adqItemPedidoCDFILIOPELOG.AsString, 'S')**/) {
                        //mensagem de erro de nao parametrizacao
                        if(!empty($adqTipoOperOL)) {
                            if(strlen($adqTipoOperOL[0]["CDFILIOPELOG"]) == 0) {
                                if(empty($cdsVdItem)) {
                                    $itemPedido["PARAMFISCOL"] = 'S';
                                    $cdsVdItem[] = $itemPedido;
                                } else {
                                    if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqTipoOperOL")) {
                                        $itemPedido["PARAMFISCOL"] = 'S';
                                        $cdsVdItem[] = $itemPedido;
                                    }
                                }
                            } else {
                                //$adsValida_Fiscal_OL = $this->service->adsValida_Fiscal_OL($itemPedido, $adqTipoOperOL[0], $stNrErFisOL);
                                $adqErroFiscalOL = $this->service->adqErroFiscalOL($itemPedido);
                                
                                if(!empty($adqErroFiscalOL) && strlen($adqErroFiscalOL[0]["CDFILIOPELOG"]) > 0) {
                                    //mensagem da especificacao
                                    if(empty($cdsVdItem)) {
                                        $itemPedido["CRTLFISOL"] = 'S';
                                        $cdsVdItem[] = $itemPedido;
                                    } else {
                                        if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqErroFiscalOL")) {
                                            $itemPedido["CRTLFISOL"] = 'S';
                                            $cdsVdItem[] = $itemPedido;
                                        }
                                    }
                                }
                            }   
                        }
                    }
                    
                    $cdsGravaEnvio[] = $itemPedido;
                }
            }
            
            foreach($cdsVdItem1 as $cdVditem1) {
                $adqProdAgrup = $this->service->adqProdAgrup($cdVditem1, $row);
                
                if(!empty($adqProdAgrup)) {
                    foreach($adqProdAgrup as $prod) {
                        if(empty($cdsVdItem)) {
                            $itemPedido["PRODAGRUP"] = 'S';
                            $cdsVdItem[] = $itemPedido;
                        } else {
                            if(!self::verificaCdsItemIgualdade($cdsVdItem, $itemPedido, "adqProdAgrup")) {
                                $itemPedido["PRODAGRUP"] = 'S';
                                $cdsVdItem[] = $itemPedido;
                            }
                        }
                    }
                }
            }
            
            $arResult[0] = $cdsVdItem;
            $arResult[1] = $cdsGravaEnvio;
            $arResult[2] = $stNrErFisOL;
            $arResult[3] = $stWebService;
            
            $result = $arResult;
            
            $response->addDataSet(new DataSet("data", $result));
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function bcconv($fNumber) {
        $sAppend = '';
        (integer)$iDecimals = ini_get('precision') - floor(log10(abs($fNumber)));
        if (0 > $iDecimals) {
            $fNumber *= pow(10, $iDecimals);
            $sAppend = str_repeat('0', -$iDecimals);
            $iDecimals = 0;
        }
        return number_format($fNumber, $iDecimals, '.', '').$sAppend;
    }
    
    public function calculoAjustes($row) {
        $dbFatoConvOrig;
        $dbFatoConvDest;
        $dbPesoUniOrig;
        $dbPesoUniDest;
        $nedAjustUn;
        $nedAjustQtde;
        
        if($row["CDPRODUTO_OLD"] != "") {
            $resultQryVdProduto = $this->service->qryVdProduto($row);
            $resultQryProduto = $this->service->qryProduto($row);
            $resultAdqAjusteQtde = $this->service->adqAjusteQtde($row);
            
            if($resultQryVdProduto["VRFATOCONV"] == "") {
                $dbFatoConvOrig = 1;
            } else {
                $dbFatoConvOrig = $resultQryVdProduto["VRFATOCONV"];
            }
            
            if($resultQryProduto["VRFATOCONV"] == "") {
                $dbFatoConvDest = 1;
            } else {
                $dbFatoConvDest = $resultQryProduto["VRFATOCONV"];
            }
            
            if($resultQryVdProduto["VRPESOUNID"] == "") {
                $dbPesoUniOrig = 1;
            } else {
                $dbPesoUniOrig = $resultQryVdProduto["VRPESOUNID"];
            }
            
            if($resultQryProduto["VRPESOUNID"] == "") {
                $dbPesoUniDest = 1;
            } else {
                $dbPesoUniDest = $resultQryProduto["VRPESOUNID"];
            }
            
            $nedAjustUn = ((($dbFatoConvOrig * $dbPesoUniOrig) / ($dbFatoConvDest * $dbPesoUniDest)) - 1) * 100;
            
            if(!empty($resultAdqAjusteQtde)) {
                foreach($resultAdqAjusteQtde as $result) {
                    if($result["CDPRODORIG"] == $resultQryVdProduto["CDPRODUTO"] && $result["CDPRODDEST"] == $resultQryProduto["CDPRODUTO"]) {
                        $nedAjustQtde = $result["VRPEAJQTDSOL"];
                    }
                }
            }
            
            if(isset($nedAjustUn)) {
                if($nedAjustUn != 0)
                    $nedAjustUn = preg_replace('/(-?\d+)\.(\d{2})\d*/', '$1, $2', $this->bcconv($nedAjustUn));
            } else {
                $nedAjustUn = NULL;
            }
            
            if(isset($nedAjustQtde)) {
                $nedAjustQtde = preg_replace('/(-?\d+)\.(\d{2})\d*/', '$1,$2', $this->bcconv($nedAjustQtde));
            } else {
                $nedAjustQtde = NULL;
            }
            
            return [ "AJUSTEUN" => $nedAjustUn, "AJUSTEQTDE" => $nedAjustQtde ];
        }
    }
    
    public function calcularAjustesProduto(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $result = $this->calculoAjustes($row);
            $response->addDataSet(new DataSet('data', [$result]));
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function adqRelacProdOL(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $adqRelacProdOL = $this->service->adqRelacProdOL($row);
            
            $response->addDataSet(new DataSet("data", $adqRelacProdOL));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function permissoesGrupoOperador(DTO\Request\Row $request, DTO\Response $response) {
        $permissoesGeraisOperador = $this->service->permissoesGeraisOperador(null);
        $permissoesFinaisOperador = $this->service->permissoesGeraisOperador($this->environment->getCdOperador());
        
        $response->addDataSet(new DataSet("permissoesGeraisOperador", $permissoesGeraisOperador));
        
        if(!empty($permissoesFinaisOperador)) {
            $response->addDataSet(new DataSet("permissoesFinaisOperador", $permissoesFinaisOperador));
        }
    }
    
    public function produtoFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $cdFilial = '';
        
        foreach($conditions as $condition) {
            if($condition["columnName"] == "CDFILIAL") {
                $cdFilial = $condition["value"];
                break;
            }
        }
        
        unset($conditions[2]);
        
        $cdSegMerc = $this->service->getCdSegMercFilial($cdFilial);
        
        if($cdSegMerc) {
            $filterSegmento = array(
                "columnName" => "CDSEGMER", 
                "operator" => "=", 
                "value" => $cdSegMerc
            );
            
            array_push($conditions, $filterSegmento);
            
            $dataSourceName = 'produto_filial_segmercado';
        } else {
            $dataSourceName = 'produtoprodcmp';
        }
    
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());

        $dataSet = new DataSet('produto_filial', $dataSet->getRows());
        $response->setDataSets(array($dataSet));
    }
    
    public function validaSegMercadoProdFili(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $cdSegMerc = $this->service->getCdSegMercFilial($row["CDFILDEST"]);
            
            if($cdSegMerc) {
                $res = $this->service->verificaSegMercado($row);
                
                if(empty($res) || count($res) == 0) {
                    throw new \Exception("PRODUTO_NOT_SEGMERCADO", 500);
                }
            }
            
            $res1 = $this->service->verificaProdFiliAtivo($row);
            
            if(empty($res1) || count($res1) == 0) {
                throw new \Exception("PRODUTO_NOT_PRODFILI_ATIVO", 500);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}